// ----------------------------------------------------------------------------
/// readinputs.c
/// © 2009-2012 Sealevel Systems, Inc.
/// <summary>
/// Sample demonstrating use of R9 Development board's digital inputs.
/// </summary>
///
/// <remarks>
/// This application acts as a simple digital input status display.
/// 
/// No command line arguments are accepted.  This tool simply displays the
/// status of all R9 digital inputs.
/// </remarks>
// ----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include "gpio.h"

#ifdef BOARD_R91001
#include "r91001_map.h"
#elif BOARD_R92100
#include "r92100_map.h"
#elif BOARD_S91X00
#include "s91x00_map.h"
#else
#error "UNKNOWN BOARD TYPE!!!!"
#endif

int main(int argc, char *args[])
{
	int i;
	int length = DIGITAL_INPUT_MAP_LENGTH;
	int state;

	// register the point(s)
	for (i = 0; i < length; i++)
	{
		if (DIGITAL_INPUT_MAP[i] < 0)
		{
			continue;
		}

		if (gpio_register(DIGITAL_INPUT_MAP[i]) != 0)
		{
			printf("Error registering input %d with sysfs!\n", i);
			return -1;
		}

		// Ensure points are configured as inputs
		if (gpio_set_direction(DIGITAL_INPUT_MAP[i], "in") != 0)
		{
			printf("Error setting direction of input %d!\n", i);
			return -2;
		}
	
		// Read the current state of each input
		if (gpio_get_value(DIGITAL_INPUT_MAP[i], &state) != 0)
		{
			printf("Error getting value of input %d!\n", i);
			return -3;
		}
		
		printf("Digital Input %3i : %i\n", i, state);

		// if you no longer wish to access the points, you can unregister them
		//gpio_unregister(DIGITAL_INPUT_MAP[i]);
	}
	
	return 0;
}
