// ----------------------------------------------------------------------------
/// toggle.c
/// © 2009-2012 Sealevel Systems, Inc.
/// <summary>
/// Sample demonstrating use of R9 Development board's digital outputs.
/// </summary>
///
/// <remarks>
/// This application acts as a simple toggle changing the state of a single
/// digital output or all digital outputs.
/// 
/// Command line arguments are accepted allowing you to specifiy the output
/// you wish to manipulate.  If no argument is specified, all outputs will 
/// be toggled.
/// </remarks>
// ----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include "gpio.h"

#ifdef BOARD_R91001
#include "r91001_map.h"
#elif BOARD_R92100
#include "r92100_map.h"
#elif BOARD_S91X00
#include "s91x00_map.h"
#else
#error "UNKNOWN BOARD TYPE!!!!"
#endif

int main(int argc, char *args[])
{
	int i, offset = 0;
	int length = DIGITAL_OUTPUT_MAP_LENGTH;

	// We will accept any argument as an indication of the
	// output we are supposed to toggle.
	if (argc > 1)
	{
		offset = atoi(args[1]);
		length = 1;
	}

	// register the point(s)
	for (i = 0; i < length; i++)
	{
		if (DIGITAL_OUTPUT_MAP[offset + i] < 0)
		{
			continue;
		}

		if (gpio_register(DIGITAL_OUTPUT_MAP[offset + i]) != 0)
		{
			printf("Error registering output %d with sysfs!\n", offset + i);
			return -1;
		}

		// Ensure points are configured as outputs
		if (gpio_set_direction(DIGITAL_OUTPUT_MAP[offset + i], "out") != 0)
		{
			printf("Error setting direction of output %d!\n", offset + i);
			return -2;
		}
	}

	// get/set state
	for (i = 0; i < length; i++)
	{
		int state;

		if (DIGITAL_OUTPUT_MAP[offset + i] < 0)
		{
			continue;
		}

		// Read the current state of each output
		if (gpio_get_value(DIGITAL_OUTPUT_MAP[offset + i], &state) != 0)
		{
			printf("Error getting value of output %d!\n", offset + i);
			return -3;
		}

		// Change the state to the opposite
		if (gpio_set_value(DIGITAL_OUTPUT_MAP[offset + i], !state) != 0)
		{
			printf("Error setting value of output %d!\n", offset + i);
			return -4;
		}
		
		printf("Digital Output %3i : %i -> %i\n", i, state, !state);
	}

	// if you no longer wish to access the points, you can unregister them
	for (i = 0; i < length; i++)
	{
		if (DIGITAL_OUTPUT_MAP[offset + i] < 0)
		{
			continue;
		}

		gpio_unregister(DIGITAL_OUTPUT_MAP[offset + i]);
	}
	
	return 0;
}
