// ----------------------------------------------------------------------------
/// recieve-async.c
/// © 2009-2012 Sealevel Systems, Inc.
/// <summary>
/// A sample program to receive async data.
/// </summary>
///
/// <remarks>
/// This is a simple example demonstrating use of a R9 for Asynchronous
/// communications.  This sample will receive (9600bps) data on the specified
/// port.  Any received data will then be written to the file "data" in the
/// executing directory.
/// </remarks>
// ----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <fcntl.h>
#include <memory.h>
#include <termios.h>
#include <unistd.h>

#include <linux/types.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>

int main(int argc, char* argv[])
{
	int fd;
	int rc;
	int disc = N_TTY;
	struct termios termios;
	FILE *fp = NULL;
	int size = 4096;
	int count;
	unsigned char *buf = malloc(size);
	char *devname;

	if (argc > 1)
		devname = argv[1];
	else
		devname = "/dev/ttyS1";

	printf("Looking for async data on %s\n", devname);

	if (!buf ) {
		printf("can't allocate buffer\n");
		return ENOMEM;
	}

	// open file to capture received data
	fp = fopen("data", "wb");
	if (fp == NULL) {
		printf("fopen(data) error=%d %s\n",
		       errno, strerror(errno));
		return errno;
	}


	// --------------------------------------------------------------------
	// open device with O_NONBLOCK to ignore DCD 
	// --------------------------------------------------------------------
	fd = open(devname, O_RDWR, 0);
	if (fd < 0) {
		printf("open of %s error=%d %s\n",
		       devname, errno, strerror(errno));
		return errno;
	}

	// set N_TTY (standard async) line discipline
	rc = ioctl(fd, TIOCSETD, &disc);
	if(rc < 0) {
		printf("ERROR, can't set line discipline error=%d %s\n",
		       errno, strerror(errno));
		return rc;
	}

	
	// --------------------------------------------------------------------
	// Configure port
	// --------------------------------------------------------------------

	// These are standard tty ldisc settings -- make sure you set these
	// if you have problems with certain characters disapearing or being
	// transformed (0xOD->0x0A, Ox11, 0x13, etc.)
	rc = tcgetattr(fd, &termios);
	if (rc < 0) {
		printf("tcgetattr() error=%d %s\n", errno, strerror(errno));
		return rc;
	}

	termios.c_iflag = 0;
	termios.c_oflag = 0;
	termios.c_cflag = CREAD | CS8 | HUPCL | CLOCAL;
	termios.c_lflag = 0;
	termios.c_cc[VTIME] = 0;
	termios.c_cc[VMIN]  = 1;
	cfsetospeed(&termios, B9600);
	cfsetispeed(&termios, B9600);

	rc = tcsetattr(fd, TCSANOW, &termios);
	if (rc < 0) {
		printf("tcsetattr() error=%d %s\n", errno, strerror(errno));
		return rc;
	}


	// --------------------------------------------------------------------
	// RX Data
	// --------------------------------------------------------------------
	for (;;) {
		memset(buf, 0, size);

		rc = read(fd, buf, size);
		if (rc < 0) {
			printf("read() error=%d %s\n",
			       errno, strerror(errno));
			break;
		}
		if (rc == 0) {
			printf("read() returned with no data\n");
			continue;
		}

		printf("received %d bytes of data\n", rc);

		// write received data to capture file
		count = fwrite(buf, sizeof(char), rc, fp);
		if (count != rc) {
			printf("data file write error=%d (%s)\n",
			       errno, strerror(errno));
			break;
		}
		fflush(fp);
	}

	return 0;
}

